Ext.namespace("dSS11", "dSS11.util");

// updateType: system/addon
dSS11.util.checkUpdateProgress = function(updateType, maskElement, doneCallback,
                                          record)
{
    var failMsg = _("Could not get current status");

    Ext.Ajax.request({
        url: '/cgi-bin/updatestatuswrapper.sh',
        disableCaching: true,
        method: 'GET',
        scope: this,
        params: { type: updateType },
        success: function(response) {
            try {
                var jsonData;
                try {
                    jsonData = Ext.util.JSON.decode(response.responseText);
                } catch (err) {
                    if (updateType === 'system') {
                        // in case of a system update the webserver might be
                        // updated as well, in this case we will not get a
                        // proper status, in some situations we might get an
                        // empty body or other unparsable things. If this
                        // happens - try again, do not error out too early.
                        var task = new Ext.util.DelayedTask(
                            dSS11.util.checkUpdateProgress, this,
                            [ updateType, maskElement, doneCallback, record ]);
                        // retry in one minute
                        task.delay(60000);
                        return;
                    } else {
                        throw err;
                    }
                }

                if (jsonData.success !== true) {
                    if (doneCallback !== undefined) {
                        doneCallback(jsonData, record);
                    } else {
                        Ext.Msg.alert(_("Error"), _(jsonData.message));
                    }
                    maskElement.unmask();
                    return;
                } else {
                    if(jsonData.code == dSS11.config.codes.ERR_DOWAIT){
                        var task = new Ext.util.DelayedTask(
                            dSS11.util.checkUpdateProgress, this,
                            [ updateType, maskElement, doneCallback, record ]);
                        task.delay(5000);
                    } else if (jsonData.code ==
                                           dSS11.config.codes.ERR_OK) {
                        maskElement.unmask();
                        if (doneCallback !== undefined) {
                            doneCallback(jsonData, record);
                        }
                    } else {
                        maskElement.unmask();
                        Ext.Msg.alert(_("Error"), _(jsonData.message));
                    }

                }
            } catch (error) {
                maskElement.unmask();
                Ext.Msg.alert(_("Error"),
                    _("Could not determine current status") + ": " +
                        _(error.message));
            }
        },
        failure: function(response) {
             if (updateType === 'system') {
                 // in case of a system update the webserver might be
                 // updated as well, in this case we will not get a
                 // proper status, in some situations we might get an
                 // empty body or other unparsable things. If this
                 // happens - try again, do not error out too early.
                 var task = new Ext.util.DelayedTask(
                     dSS11.util.checkUpdateProgress, this,
                     [ updateType, maskElement, doneCallback, record ]);
                 // retry in one minute
                 task.delay(60000);
                 return;
             }
             maskElement.unmask();
             Ext.Msg.alert(_("Error"), failMsg);
        }
    });
};

dSS11.util.dSMFW = function(action, maskElement, doneCallback)
{
    var message, msgFail1, msgFail2;

    if (action === 'check') {
        message = _("Checking if dSM firmware is up to date");
        msgFail1 = _("Could not check if dSMs need update");
        msgFail2 = msgFail1;
    } else if (action === 'upgrade') {
        message = _("Upgrading dSM firmware, this may take several minutes");
        msgFail1 = _("Could not upgrade dSM firmware");
        msgFail2 = _("Could not start upgrade of dSM firmware");
    } else {
        throw _("invalid action");
    }

    if (maskElement !== undefined) {
        maskElement.mask(message + "...", 'x-mask-loading');
    }

    var errcb = function() {
        if ((action === 'check') && (doneCallback !== undefined)) {
            doneCallback(-1);
        }
    };

    Ext.Ajax.request({
        url: '/cgi-bin/dsmfwwrapper.sh',
        disableCaching: true,
        method: 'GET',
        scope: this,
        params: { action: action },
        success: function(response)
        {
            try {
                var jsonData = Ext.util.JSON.decode(response.responseText);
                if (jsonData.success !== true) {
                    Ext.Msg.alert(_("Error"), _(jsonData.message), errcb);
                    if (maskElement !== undefined) {
                        maskElement.unmask();
                    }
                    return;
                }

                if (action === 'check') {
                    if (maskElement !== undefined) {
                        maskElement.unmask();
                    }
                    if (doneCallback !== undefined) {
                        doneCallback(jsonData.data.update);
                    }
                } else {
                    var fn = dSS11.util.checkUpdateProgress.createDelegate(
                                        this,
                                        [
                                            'dsmfwupgrade',
                                            maskElement,
                                            doneCallback
                                        ]);
                    fn();
                }
            } catch (error) {
                Ext.Msg.alert(_("Error"), msgFail1 + ": " + _(error.message),
                              errcb);
                if (maskElement !== undefined) {
                    maskElement.unmask();
                }
            }
        },
        failure: function(response)
        {
            Ext.Msg.alert(_("Error"), msgFail2, errcb);
            if (maskElement !== undefined) {
                maskElement.unmask();
            }
        }
    });
};

dSS11.util.checkResponse = function(response)
{
    var json = Ext.util.JSON.decode(response.responseText);
    if (json.success !== true) {
        throw json.message;
    }
};


dSS11.util.showDisclaimer = function(load, maskElement)
{
    if (maskElement !== undefined) {
        maskElement.mask();
    }
    var win = new Ext.Window(
    {
        title: _("Disclaimer"),
        width: 640,
        height: 480,
        layout: 'fit',
        listeners:
        {
            close: function()
            {
                if (maskElement !== undefined) {
                    maskElement.unmask();
                }
            }
        },
        items:
        [
            {
                xtype: 'panel',
                border: false,
                autoScroll: true,
                autoLoad: '/disclaimer.html'
            }
        ],
        buttons:
        [
            {
                text: _("Accept"),
                handler: function()
                {
                    Ext.Ajax.request({
                        url: '/cgi-bin/acceptdscwrapper.sh',
                        method: 'GET',
                        params: { accept: 'yes' },
                        success: function(response) {
                            try {
                                dSS11.util.checkResponse(response);
                                dSS11.StatusBar.setMessage(
                                    _("Disclaimer accepted"));
                            } catch (err) {
                                Ext.Msg.alert(_("Error"),
                                    _("Could not apply disclaimer setting") +
                                        ': ' + err);
                            }
                            win.close();
                            if (maskElement !== undefined) {
                                maskElement.unmask();
                            }
                        },
                        failure: function(response) {
                            try {
                                dSS11.util.checkResponse(response);
                            } catch (err) {
                                Ext.Msg.alert(_("Error"),
                                    _("Could not apply disclaimer setting") +
                                        ': ' + err);
                            }
                            win.close();
                            if (maskElement !== undefined) {
                                maskElement.unmask();
                            }
                        }
                    });
                }
            },
            {
                text: _("Close"),
                scope: this,
                handler: function() {
                    win.close();
                    if (maskElement !== undefined) {
                        maskElement.unmask();
                    }
                }
            }
        ]
    });
    win.show(this);
};

dSS11.util.opkg = function(action, maskElement, doneCallback, record)
{
    var message = _("Please wait");

    var name;
    var hrname;
    if (record !== undefined) {
        name = record.get('name');
        hrname = record.get('hrname');
    }

    if ((action === 'update') || (action === 'addon-update')) {
        message = _("Downloading package lists from server");
    } else if ((action === 'upgrade') || (action === 'addon-upgrade')) {
        message = _("Performing update, this may take several minutes");
    } else if (action === 'addon-install') {
        message = _("Please wait") + ', ' +  _("app") + ' <b>' + hrname +
                    '</b>' + ' ' + _("is being installed");
    } else if (action === 'addon-remove') {
        message = _("Please wait") + ', ' + _("app") + ' ' +
                    ' <b>' + hrname + '</b>' + ' ' + _("is being removed");
    }
    maskElement.mask(message + "...",
                     'x-mask-loading');

    Ext.Ajax.request({
        url: '/cgi-bin/opkgwrapper.sh',
        disableCaching: true,
        method: 'GET',
        scope: this,
        params: { action: action, name: name },
        success: function(response)
        {
            try {
                    var jsonData = Ext.util.JSON.decode(
                                                response.responseText);
                    if (jsonData.success !== true) {
                        maskElement.unmask();
                        if (jsonData.code ===
                                            dSS11.config.codes.ERR_DISCLAIMER) {
                            dSS11.util.showDisclaimer(false, maskElement);
                            return;
                        }
                        Ext.Msg.alert(_("Error"), _(jsonData.message));
                        return;
                    } else {
                        var type = 'system';
                        if (action.slice(0, 5) === 'addon') {
                            type = 'addon';
                        }
                        var fn = dSS11.util.checkUpdateProgress.createDelegate(
                                            this,
                                            [
                                                type,
                                                maskElement,
                                                doneCallback,
                                                record
                                            ]);
                        fn();
                    }
            } catch (error) {
                maskElement.unmask();
                Ext.Msg.alert(_("Error"),
                    _("Failed to download package lists from the server") +
                    ": " + _(error.message));
            }
        },
        failure: function() {
             maskElement.unmask();
             Ext.Msg.alert(_("Error"), _("Could not request new package" +
                           " lists"));
        }
    });
};

dSS11.util.setApartmentName = function(name) {
    if (dSS11.config.dSSIntegration === true) {
        Ext.Ajax.request({
            url: 'json/apartment/setName',
            method: 'GET',
            scope: this,
            params: { newName: name }
        });
    }
};

dSS11.util.defaultAddOnIconRenderer = function(value, p, record) {
    return '<span><img width="48" height="48" ' +
           'src="/images/default_icon.png"/></span>';
};

dSS11.util.defaultIPKTemplate = function() {
    return new Ext.XTemplate(
                '<tpl for=".">',
                  '<div class="x-black-text {css}">',
                      '<b>',
                      '<tpl if="((hrname !== undefined) && (hrname.length !== 0))">',
                      '{hrname}',
                      '</tpl>',
                      '<tpl if="((hrname === undefined) || (hrname.length === 0))">',
                      '{name}',
                      '</tpl>',
                      '</b>',
                      '<tpl if="homepage !== undefined">',
                          '<span class="x-text-link">',
                            '&nbsp;<a href="{homepage}" target="_blank">',
                            '{homepage}</a>',
                          '</span>',
                      '</tpl>',
                      '<br/>',
                      _("Version") + ':&nbsp;{version}<br/>',
                      '<tpl if="((description === undefined) || (description.length === 0)) ">',
                      '<br/>',
                      '</tpl>',
                      '{description}',
                    '</div>',
                '</tpl>',
                '<div class="x-clear"></div>'
            );
};

dSS11.util.validateIP = function(ip) {
        var i;

        if (ip === undefined) {
            return false;
        }

        ip = ip.replace(/\s/g, "");
        var re = /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/;
        if (re.test(ip)) {
            var parts = ip.split(".");
            if (parts.lengh < 4) {
                return false;
            }

            var n1 = parseInt(parseFloat(parts[0]), 10);
            var n2 = parseInt(parseFloat(parts[1]), 10);
            var n3 = parseInt(parseFloat(parts[2]), 10);
            var n4 = parseInt(parseFloat(parts[3]), 10);

            if (n1 === n2 === n3 === n4 === 255) {
                return false;
            }

            if (n1 === n2 === n3 === n4 === 0) {
                return false;
            }

            if (n1 === 0) {
                return false;
            }

            if (n4 === 255) {
                return false;
            }

            for (i = 0; i < parts.length; i++) {
                var num = parseInt(parseFloat(parts[i]), 10);
                if ((num > 255) || (num < 0)) {
                    return false;
                }
            }
            return true;
        } else {
            return false;
        }
};

dSS11.util.rmLeadingZeroes = function(s) {
    var i = 0;
    while (s[i] === '0') {
        i++;
    }
    return s.substr(i);
};

dSS11.util.toFixed = function(val, precision) {
    var p = Math.pow(10, precision || 0);
    return Math.round(val * p) / p;
};

dSS11.util.decodeHTMLentity = function(s) {
    var temp = document.createElement("textarea");
    temp.innerHTML=s.replace(/</g,"&lt;").replace(/>/g,"&gt;");
    return temp.value;
};
